@@ -3,7 +3,7 @@ package Alien::Base::ModuleBuild;
 use strict;
 use warnings;
 
-our $VERSION = '0.018';
+our $VERSION = '0.019';
 $VERSION = eval $VERSION;
 
 use parent 'Module::Build';
@@ -130,6 +130,13 @@ __PACKAGE__->add_property( 'alien_bin_requires' => {} );
 # Do a staged install to blib instead of trying to install to the final location.
 __PACKAGE__->add_property( 'alien_stage_install' => 1 );
 
+# Should modules be installed into arch specific directory?
+# Most alien dists will have arch specific files in their share so it makes sense to install
+# the module in the arch specific location.  If you are alienizing something that isn't arch
+# specific, like javascript source or java byte code, then you'd want to set this to 0.
+# For now this will default to off.  See gh#119 for a discussion.
+__PACKAGE__->add_property( 'alien_arch' => defined $ENV{ALIEN_ARCH} ? $ENV{ALIEN_ARCH} : 0 );
+
 ################
 #  ConfigData  #
 ################
@@ -283,6 +290,17 @@ sub Inline { shift; $module->Inline(\@_) }
 EOF
     close $fh;
   }
+  
+  if($self->alien_arch) {
+    my @parts = split /::/, $module;
+    my $arch_dir = File::Spec->catdir($self->blib, 'arch', 'auto', @parts);
+    File::Path::mkpath($arch_dir, 0, oct(777)) unless -d $arch_dir;
+    open my $fh, '>', File::Spec->catfile($arch_dir, $parts[-1].".txt");
+    print $fh "Alien based distribution with architecture specific file in share\n";
+    close $fh;
+  }
+
+  $self->depends_on('alien_install') if $self->alien_stage_install;
 }
 
 sub process_share_dir_files {
@@ -576,7 +594,7 @@ sub alien_library_destination {
   my $lib_dir = 
     $self->notes('alien_blib_scheme') || $self->alien_stage_install
     ? File::Spec->catdir( $self->base_dir, $self->blib, 'lib' )
-    : $self->install_destination('lib');
+    : $self->install_destination($self->alien_arch ? 'arch' : 'lib');
 
   my $dist_name = $self->dist_name;
   my $dest = _catdir( $lib_dir, qw/auto share dist/, $dist_name );
@@ -1025,7 +1043,7 @@ sub alien_relocation_fixup {
   return unless $^O eq 'darwin';
 
   my $dist_name = $self->dist_name;  
-  my $share = _catdir( $self->install_destination('lib'), qw/auto share dist/, $dist_name );
+  my $share = _catdir( $self->install_destination($self->alien_arch ? 'arch' : 'lib'), qw/auto share dist/, $dist_name );
   
   require File::Find;
   File::Find::find(sub {
